"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const locale_1 = require("../../locale");
const shim_1 = require("../../shim");
const BaseItem_1 = require("../BaseItem");
const Resource_1 = require("../Resource");
const Setting_1 = require("../Setting");
const validations = {
    'sync.target': (oldValue, newValue) => __awaiter(void 0, void 0, void 0, function* () {
        if (oldValue === 0 || newValue === 0)
            return '';
        const preventChangeMessage = () => __awaiter(void 0, void 0, void 0, function* () {
            const needToBeFetched = yield Resource_1.default.needToBeFetched('always', 1);
            if (needToBeFetched.length)
                return (0, locale_1._)('Some attachments need to be downloaded. Set the attachment download mode to "always" and try again.');
            const downloadErrorCount = yield Resource_1.default.downloadStatusCounts(Resource_1.default.FETCH_STATUS_ERROR);
            if (downloadErrorCount > 0)
                return (0, locale_1._)('Some attachments could not be downloaded. Please try to download them again.');
            const disabledCount = yield BaseItem_1.default.syncDisabledItemsCount(Setting_1.default.value('sync.target'));
            if (disabledCount > 0)
                return (0, locale_1._)('Some items could not be synchronised. Please try to synchronise them first.');
            return '';
        });
        const message = yield preventChangeMessage();
        if (message) {
            const resolutionMessage = shim_1.default.isReactNative() ?
                (0, locale_1._)('Uninstall and reinstall the application. Make sure you create a backup first by exporting all your notes as JEX from the desktop application.') :
                (0, locale_1._)('Close the application, then delete your profile in "%s", and start the application again. Make sure you create a backup first by exporting all your notes as JEX.', Setting_1.default.value('profileDir'));
            return (0, locale_1._)('The sync target cannot be changed for the following reason: %s\n\nIf the issue cannot be resolved, you may need to clear your data first by following these instructions:\n\n%s', message, resolutionMessage);
        }
        return '';
    }),
};
const validateSetting = (settingName, oldValue, newValue) => __awaiter(void 0, void 0, void 0, function* () {
    if (oldValue === newValue)
        return '';
    if (!validations[settingName])
        return '';
    return yield validations[settingName](oldValue, newValue);
});
exports.default = (settingKeys, newValues) => __awaiter(void 0, void 0, void 0, function* () {
    for (const key of settingKeys) {
        const oldValue = Setting_1.default.value(key);
        const newValue = newValues[key];
        const message = yield validateSetting(key, oldValue, newValue);
        return message;
    }
    return '';
});
//# sourceMappingURL=settingValidations.js.map