"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("../../testing/test-utils");
const SearchEngine_1 = require("./SearchEngine");
const Note_1 = require("../../models/Note");
const ItemChange_1 = require("../../models/ItemChange");
const Setting_1 = require("../../models/Setting");
let engine = null;
// const IDF = (N:number, n:number) => Math.max(Math.log((N - n + 0.5) / (n + 0.5)), 0);
// const frequency = (word:string, string:string) => {
// 	const re = new RegExp(`\\b(${word})\\b`, 'g');
// 	return (string.match(re) || []).length;
// };
// const calculateScore = (searchString, notes) => {
// 	const K1 = 1.2;
// 	const B = 0.75;
// 	const freqTitle = notes.map(note => frequency(searchString, note.title));
// 	const notesWithWord = freqTitle.filter(count => count !== 0).length;
// 	const numTokens = notes.map(note => note.title.split(' ').length);
// 	const avgTokens = Math.round(numTokens.reduce((a, b) => a + b, 0) / notes.length);
// 	const msSinceEpoch = Math.round(new Date().getTime());
// 	const msPerDay = 86400000;
// 	const weightForDaysSinceLastUpdate = (row) => {
// 		// BM25 weights typically range 0-10, and last updated date should weight similarly, though prioritizing recency logarithmically.
// 		// An alpha of 200 ensures matches in the last week will show up front (11.59) and often so for matches within 2 weeks (5.99),
// 		// but is much less of a factor at 30 days (2.84) or very little after 90 days (0.95), focusing mostly on content at that point.
// 		if (!row.user_updated_time) {
// 			return 0;
// 		}
// 		const alpha = 200;
// 		const daysSinceLastUpdate = (msSinceEpoch - row.user_updated_time) / msPerDay;
// 		return alpha * Math.log(1 + 1 / Math.max(daysSinceLastUpdate, 0.5));
// 	};
// 	let titleBM25WeightedByLastUpdate = new Array(notes.length).fill(-1);
// 	if (avgTokens !== 0) {
// 		for (let i = 0; i < notes.length; i++) {
// 			titleBM25WeightedByLastUpdate[i] = IDF(notes.length, notesWithWord) * ((freqTitle[i] * (K1 + 1)) / (freqTitle[i] + K1 * (1 - B + B * (numTokens[i] / avgTokens))));
// 			titleBM25WeightedByLastUpdate[i] += weightForDaysSinceLastUpdate(notes[i]);
// 		}
// 	}
// 	const scores = [];
// 	for (let i = 0; i < notes.length; i++) {
// 		if (freqTitle[i]) scores.push(titleBM25WeightedByLastUpdate[i]);
// 	}
// 	scores.sort().reverse();
// 	return scores;
// };
describe('services/SearchEngine', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
        engine = new SearchEngine_1.default();
        engine.setDb((0, test_utils_1.db)());
    }));
    it('should keep the content and FTS table in sync', (() => __awaiter(void 0, void 0, void 0, function* () {
        let rows;
        const n1 = yield Note_1.default.save({ title: 'a' });
        const n2 = yield Note_1.default.save({ title: 'b' });
        yield engine.syncTables();
        rows = yield engine.search('a');
        expect(rows.length).toBe(1);
        expect(rows[0].title).toBe('a');
        yield Note_1.default.delete(n1.id);
        yield engine.syncTables();
        rows = yield engine.search('a');
        expect(rows.length).toBe(0);
        rows = yield engine.search('b');
        expect(rows[0].title).toBe('b');
        yield Note_1.default.save({ id: n2.id, title: 'c' });
        yield engine.syncTables();
        rows = yield engine.search('b');
        expect(rows.length).toBe(0);
        rows = yield engine.search('c');
        expect(rows[0].title).toBe('c');
        yield Note_1.default.save({ id: n2.id, encryption_applied: 1 });
        yield engine.syncTables();
        rows = yield engine.search('c');
        expect(rows.length).toBe(0);
        yield Note_1.default.save({ id: n2.id, encryption_applied: 0 });
        yield engine.syncTables();
        rows = yield engine.search('c');
        expect(rows.length).toBe(1);
    })));
    it('should, after initial indexing, save the last change ID', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: 'abcd efgh' }); // 3
        yield Note_1.default.save({ title: 'abcd aaaaa abcd abcd' }); // 1
        expect(Setting_1.default.value('searchEngine.initialIndexingDone')).toBe(false);
        yield ItemChange_1.default.waitForAllSaved();
        const lastChangeId = yield ItemChange_1.default.lastChangeId();
        yield engine.syncTables();
        expect(Setting_1.default.value('searchEngine.lastProcessedChangeId')).toBe(lastChangeId);
        expect(Setting_1.default.value('searchEngine.initialIndexingDone')).toBe(true);
    })));
    it('should order search results by relevance BM25', (() => __awaiter(void 0, void 0, void 0, function* () {
        // BM25 is based on term frequency - inverse document frequency
        // The tf–idf value increases proportionally to the number of times a word appears in the document
        // and is offset by the number of documents in the corpus that contain the word, which helps to adjust
        // for the fact that some words appear more frequently in general.
        // BM25 returns weight zero for search term which occurs in more than half the notes.
        // So terms that are abundant in all notes to have zero relevance w.r.t BM25.
        const n1 = yield Note_1.default.save({ title: 'abcd efgh' }); // 3
        const n2 = yield Note_1.default.save({ title: 'abcd efgh abcd abcd' }); // 1
        const n3 = yield Note_1.default.save({ title: 'abcd aaaaa bbbb eeee abcd' }); // 2
        yield Note_1.default.save({ title: 'xyz xyz' });
        yield Note_1.default.save({ title: 'xyz xyz xyz xyz' });
        yield Note_1.default.save({ title: 'xyz xyz xyz xyz xyz xyz' });
        yield Note_1.default.save({ title: 'xyz xyz xyz xyz xyz xyz' });
        yield Note_1.default.save({ title: 'xyz xyz xyz xyz xyz xyz xyz xyz' });
        yield engine.syncTables();
        let rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n2.id);
        expect(rows[1].id).toBe(n3.id);
        expect(rows[2].id).toBe(n1.id);
        rows = yield engine.search('abcd efgh');
        expect(rows[0].id).toBe(n1.id); // shorter note; also 'efgh' is more rare than 'abcd'.
        expect(rows[1].id).toBe(n2.id);
    })));
    it('should order search results by relevance BM25 - 2', () => __awaiter(void 0, void 0, void 0, function* () {
        // This simple test case didn't even work before due to a bug in the IDF
        // calculation, and would just order by timestamp.
        const n1 = yield Note_1.default.save({ title: 'abcd abcd' }); // 1
        yield (0, test_utils_1.msleep)(1);
        const n2 = yield Note_1.default.save({ title: 'abcd' }); // 2
        yield engine.syncTables();
        const rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n1.id);
        expect(rows[1].id).toBe(n2.id);
    }));
    // TODO: Need to update and replace jasmine.mockDate() calls with Jest
    // equivalent
    // it('should correctly weigh notes using BM25 and user_updated_time', (async () => {
    // 	await mockDate(2020, 9, 30, 50);
    // 	const noteData = [
    // 		{
    // 			title: 'abc test2 test2',
    // 			updated_time: 1601425064756,
    // 			user_updated_time: 1601425064756,
    // 			created_time: 1601425064756,
    // 			user_created_time: 1601425064756,
    // 		},
    // 		{
    // 			title: 'foo foo',
    // 			updated_time: 1601425064758,
    // 			user_updated_time: 1601425064758,
    // 			created_time: 1601425064758,
    // 			user_created_time: 1601425064758,
    // 		},
    // 		{
    // 			title: 'dead beef',
    // 			updated_time: 1601425064760,
    // 			user_updated_time: 1601425064760,
    // 			created_time: 1601425064760,
    // 			user_created_time: 1601425064760,
    // 		},
    // 		{
    // 			title: 'test2 bar',
    // 			updated_time: 1601425064761,
    // 			user_updated_time: 1601425064761,
    // 			created_time: 1601425064761,
    // 			user_created_time: 1601425064761,
    // 		},
    // 		{
    // 			title: 'blah blah abc',
    // 			updated_time: 1601425064763,
    // 			user_updated_time: 1601425064763,
    // 			created_time: 1601425064763,
    // 			user_created_time: 1601425064763,
    // 		},
    // 	];
    // 	const n0 = await Note.save(noteData[0], { autoTimestamp: false });
    // 	const n1 = await Note.save(noteData[1], { autoTimestamp: false });
    // 	const n2 = await Note.save(noteData[2], { autoTimestamp: false });
    // 	const n3 = await Note.save(noteData[3], { autoTimestamp: false });
    // 	const n4 = await Note.save(noteData[4], { autoTimestamp: false });
    // 	restoreDate();
    // 	await engine.syncTables();
    // 	await mockDate(2020, 9, 30, 50);
    // 	let searchString = 'abc';
    // 	let scores = calculateScore(searchString, noteData);
    // 	let rows = await engine.search(searchString);
    // 	expect(rows[0].weight).toEqual(scores[0]);
    // 	expect(rows[1].weight).toEqual(scores[1]);
    // 	// console.log(rows);
    // 	// console.log(scores);
    // 	searchString = 'test2';
    // 	scores = calculateScore(searchString, noteData);
    // 	rows = await engine.search(searchString);
    // 	// console.log(rows);
    // 	// console.log(scores);
    // 	expect(rows[0].weight).toEqual(scores[0]);
    // 	expect(rows[1].weight).toEqual(scores[1]);
    // 	searchString = 'foo';
    // 	scores = calculateScore(searchString, noteData);
    // 	rows = await engine.search(searchString);
    // 	// console.log(rows);
    // 	// console.log(scores);
    // 	expect(rows[0].weight).toEqual(scores[0]);
    // 	await restoreDate();
    // }));
    it('should tell where the results are found', (() => __awaiter(void 0, void 0, void 0, function* () {
        const notes = [
            yield Note_1.default.save({ title: 'abcd efgh', body: 'abcd' }),
            yield Note_1.default.save({ title: 'abcd' }),
            yield Note_1.default.save({ title: 'efgh', body: 'abcd' }),
        ];
        yield engine.syncTables();
        const testCases = [
            ['abcd', ['title', 'body'], ['title'], ['body']],
            ['efgh', ['title'], [], ['title']],
        ];
        for (const testCase of testCases) {
            const rows = yield engine.search(testCase[0]);
            for (let i = 0; i < notes.length; i++) {
                const row = rows.find(row => row.id === notes[i].id);
                const actual = row ? row.fields.sort().join(',') : '';
                const expected = testCase[i + 1].sort().join(',');
                expect(expected).toBe(actual);
            }
        }
    })));
    it('should order search results by relevance (last updated first)', (() => __awaiter(void 0, void 0, void 0, function* () {
        let rows;
        const n1 = yield Note_1.default.save({ title: 'abcd' });
        yield (0, test_utils_1.sleep)(0.1);
        const n2 = yield Note_1.default.save({ title: 'abcd' });
        yield (0, test_utils_1.sleep)(0.1);
        const n3 = yield Note_1.default.save({ title: 'abcd' });
        yield (0, test_utils_1.sleep)(0.1);
        yield engine.syncTables();
        rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n3.id);
        expect(rows[1].id).toBe(n2.id);
        expect(rows[2].id).toBe(n1.id);
        yield Note_1.default.save({ id: n1.id, title: 'abcd' });
        yield engine.syncTables();
        rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n1.id);
        expect(rows[1].id).toBe(n3.id);
        expect(rows[2].id).toBe(n2.id);
    })));
    it('should order search results by relevance (completed to-dos last)', (() => __awaiter(void 0, void 0, void 0, function* () {
        let rows;
        const n1 = yield Note_1.default.save({ title: 'abcd', is_todo: 1 });
        yield (0, test_utils_1.sleep)(0.1);
        const n2 = yield Note_1.default.save({ title: 'abcd', is_todo: 1 });
        yield (0, test_utils_1.sleep)(0.1);
        const n3 = yield Note_1.default.save({ title: 'abcd', is_todo: 1 });
        yield (0, test_utils_1.sleep)(0.1);
        yield engine.syncTables();
        rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n3.id);
        expect(rows[1].id).toBe(n2.id);
        expect(rows[2].id).toBe(n1.id);
        yield Note_1.default.save({ id: n3.id, todo_completed: Date.now() });
        yield engine.syncTables();
        rows = yield engine.search('abcd');
        expect(rows[0].id).toBe(n2.id);
        expect(rows[1].id).toBe(n1.id);
        expect(rows[2].id).toBe(n3.id);
    })));
    it('should support searching through documents that contain null characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({
            title: 'Test',
            body: `
				NUL characters, "\x00", have been known to break FTS search.
				Previously, all characters after a NUL (\x00) character in a note
				would not show up in search results. NUL characters may have also
				broken search for other notes.

				In this note, "testing" only appears after the NUL characters.
			`,
        });
        yield engine.syncTables();
        expect((yield engine.search('previously')).length).toBe(1);
        expect((yield engine.search('testing')).length).toBe(1);
    })));
    it('should supports various query types', (() => __awaiter(void 0, void 0, void 0, function* () {
        let rows;
        yield Note_1.default.save({ title: 'abcd efgh ijkl', body: 'aaaa bbbb' });
        yield Note_1.default.save({ title: 'iiii efgh bbbb', body: 'aaaa bbbb' });
        yield Note_1.default.save({ title: 'Агентство Рейтер' });
        yield Note_1.default.save({ title: 'Dog' });
        yield Note_1.default.save({ title: 'СООБЩИЛО' });
        yield engine.syncTables();
        rows = yield engine.search('abcd ijkl');
        expect(rows.length).toBe(1);
        rows = yield engine.search('"abcd ijkl"');
        expect(rows.length).toBe(0);
        rows = yield engine.search('"abcd efgh"');
        expect(rows.length).toBe(1);
        rows = yield engine.search('title:abcd');
        expect(rows.length).toBe(1);
        rows = yield engine.search('title:efgh');
        expect(rows.length).toBe(2);
        rows = yield engine.search('body:abcd');
        expect(rows.length).toBe(0);
        rows = yield engine.search('body:bbbb');
        expect(rows.length).toBe(2);
        rows = yield engine.search('body:bbbb iiii');
        expect(rows.length).toBe(1);
        rows = yield engine.search('Рейтер');
        expect(rows.length).toBe(1);
        rows = yield engine.search('рейтер');
        expect(rows.length).toBe(1);
        rows = yield engine.search('Dog');
        expect(rows.length).toBe(1);
        rows = yield engine.search('dog');
        expect(rows.length).toBe(1);
        rows = yield engine.search('сообщило');
        expect(rows.length).toBe(1);
    })));
    it('should support queries with or without accents', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: 'père noël' });
        yield engine.syncTables();
        expect((yield engine.search('père')).length).toBe(1);
        expect((yield engine.search('pere')).length).toBe(1);
        expect((yield engine.search('noe*')).length).toBe(1);
        expect((yield engine.search('noë*')).length).toBe(1);
    })));
    it('should support queries with Chinese characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: '我是法国人', body: '中文测试' });
        yield engine.syncTables();
        expect((yield engine.search('我')).length).toBe(1);
        expect((yield engine.search('法国人')).length).toBe(1);
        expect((yield engine.search('法国人*'))[0].fields.sort()).toEqual(['body', 'title']); // usually assume that keyword was matched in body
        expect((yield engine.search('测试')).length).toBe(1);
        expect((yield engine.search('测试'))[0].fields).toEqual(['body']);
        expect((yield engine.search('测试*'))[0].fields).toEqual(['body']);
        expect((yield engine.search('any:1 type:todo 测试')).length).toBe(1);
    })));
    it('should support queries with Japanese characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: '私は日本語を話すことができません', body: 'テスト' });
        yield engine.syncTables();
        expect((yield engine.search('日本')).length).toBe(1);
        expect((yield engine.search('できません')).length).toBe(1);
        expect((yield engine.search('できません*'))[0].fields.sort()).toEqual(['body', 'title']); // usually assume that keyword was matched in body
        expect((yield engine.search('テスト'))[0].fields.sort()).toEqual(['body']);
        expect((yield engine.search('any:1 type:todo テスト')).length).toBe(1);
    })));
    it('should support queries with Korean characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: '이것은 한국말이다' });
        yield engine.syncTables();
        expect((yield engine.search('이것은')).length).toBe(1);
        expect((yield engine.search('말')).length).toBe(1);
        expect((yield engine.search('any:1 type:todo 말')).length).toBe(1);
    })));
    it('should support queries with Thai characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: 'นี่คือคนไทย' });
        yield engine.syncTables();
        expect((yield engine.search('นี่คือค')).length).toBe(1);
        expect((yield engine.search('ไทย')).length).toBe(1);
        expect((yield engine.search('any:1 type:todo ไทย')).length).toBe(1);
    })));
    it('should parse normal query strings', (() => __awaiter(void 0, void 0, void 0, function* () {
        const testCases = [
            ['abcd efgh', { _: ['abcd', 'efgh'] }],
            ['abcd   efgh', { _: ['abcd', 'efgh'] }],
            ['title:abcd efgh', { _: ['efgh'], title: ['abcd'] }],
            ['title:abcd', { title: ['abcd'] }],
            ['"abcd efgh"', { _: ['abcd efgh'] }],
            ['"abcd efgh" ijkl', { _: ['abcd efgh', 'ijkl'] }],
            ['title:abcd title:efgh', { title: ['abcd', 'efgh'] }],
        ];
        for (let i = 0; i < testCases.length; i++) {
            const t = testCases[i];
            const input = t[0];
            const expected = t[1];
            const actual = yield engine.parseQuery(input);
            const _Values = actual.terms._ ? actual.terms._.map((v) => v.value) : undefined;
            const titleValues = actual.terms.title ? actual.terms.title.map((v) => v.value) : undefined;
            const bodyValues = actual.terms.body ? actual.terms.body.map((v) => v.value) : undefined;
            expect(JSON.stringify(_Values)).toBe(JSON.stringify(expected._));
            expect(JSON.stringify(titleValues)).toBe(JSON.stringify(expected.title));
            expect(JSON.stringify(bodyValues)).toBe(JSON.stringify(expected.body));
        }
    })));
    it('should handle queries with special characters', (() => __awaiter(void 0, void 0, void 0, function* () {
        let rows;
        const testCases = [
            // "-" is considered a word delimiter so it is stripped off
            // when indexing the notes. "did-not-match" is translated to
            // three word "did", "not", "match"
            ['did-not-match', 'did not match'],
            ['did-not-match', '"did-not-match"'],
            ['does match', 'does match'],
        ];
        for (let i = 0; i < testCases.length; i++) {
            const t = testCases[i];
            const content = t[0];
            const query = t[1];
            const n = yield Note_1.default.save({ title: content });
            yield engine.syncTables();
            rows = yield engine.search(query);
            expect(rows.length).toBe(1);
            yield Note_1.default.delete(n.id);
        }
    })));
    it('should allow using basic search', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: '- [ ] abcd' });
        yield Note_1.default.save({ title: '[ ] abcd' });
        yield engine.syncTables();
        expect((yield engine.search('"- [ ]"', { searchType: SearchEngine_1.default.SEARCH_TYPE_FTS })).length).toBe(0);
        expect((yield engine.search('"- [ ]"', { searchType: SearchEngine_1.default.SEARCH_TYPE_BASIC })).length).toBe(1);
        expect((yield engine.search('"[ ]"', { searchType: SearchEngine_1.default.SEARCH_TYPE_BASIC })).length).toBe(2);
    })));
    it('should not mistake cyrillic "l" for latin "n"', (() => __awaiter(void 0, void 0, void 0, function* () {
        const n1 = yield Note_1.default.save({ title: 'latin n', body: 'n' });
        const n2 = yield Note_1.default.save({ title: 'cyrillic l', body: 'л' });
        yield engine.syncTables();
        expect((yield engine.search('n')).length).toBe(1);
        expect((yield engine.search('n'))[0].id).toBe(n1.id);
        expect((yield engine.search('л')).length).toBe(1);
        expect((yield engine.search('л'))[0].id).toBe(n2.id);
    })));
    it('should automatically add wildcards', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: 'hello1' });
        yield Note_1.default.save({ title: 'hello2' });
        yield engine.syncTables();
        expect((yield engine.search('hello')).length).toBe(0);
        expect((yield engine.search('hello', { appendWildCards: true })).length).toBe(2);
    })));
    it('should search HTML-entity encoded text', (() => __awaiter(void 0, void 0, void 0, function* () {
        yield Note_1.default.save({ title: '&#xE9;&#xE7;&#xE0;' }); // éçà
        yield engine.syncTables();
        const rows = yield engine.search('éçà');
        expect(rows.length).toBe(1);
    })));
    // Disabled for now:
    // https://github.com/laurent22/joplin/issues/9769#issuecomment-1912459744
    // it('should search by item ID if no other result was found', (async () => {
    // 	const f1 = await Folder.save({});
    // 	const n1 = await Note.save({ title: 'hello1', parent_id: f1.id });
    // 	const n2 = await Note.save({ title: 'hello2' });
    // 	await engine.syncTables();
    // 	const results = await engine.search(n1.id);
    // 	expect(results.length).toBe(1);
    // 	expect(results[0].id).toBe(n1.id);
    // 	expect(results[0].title).toBe(n1.title);
    // 	expect(results[0].parent_id).toBe(n1.parent_id);
    // 	expect((await engine.search(n2.id))[0].id).toBe(n2.id);
    // 	expect(await engine.search(f1.id)).toEqual([]);
    // }));
});
//# sourceMappingURL=SearchEngine.test.js.map