"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Resource_1 = require("../../../models/Resource");
const BaseItem_1 = require("../../../models/BaseItem");
const Setting_1 = require("../../../models/Setting");
const Logger_1 = require("@joplin/utils/Logger");
const logger = Logger_1.default.create('checkDisabledSyncItemsNotification');
exports.default = (dispatch) => __awaiter(void 0, void 0, void 0, function* () {
    const errorCount = yield Resource_1.default.downloadStatusCounts(Resource_1.default.FETCH_STATUS_ERROR);
    if (errorCount) {
        logger.info(`${errorCount} resource download errors: Triggering notification`);
        dispatch({ type: 'SYNC_HAS_DISABLED_SYNC_ITEMS' });
        return;
    }
    const disabledCount = yield BaseItem_1.default.syncDisabledItemsCount(Setting_1.default.value('sync.target'));
    if (disabledCount) {
        logger.info(`${disabledCount} disabled sync items: Triggering notification`);
        dispatch({ type: 'SYNC_HAS_DISABLED_SYNC_ITEMS' });
        return;
    }
    logger.info('No errors: Hiding notification');
    dispatch({
        type: 'SYNC_HAS_DISABLED_SYNC_ITEMS',
        value: false,
    });
});
//# sourceMappingURL=checkDisabledSyncItemsNotification.js.map