"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("@joplin/utils/Logger");
const shim_1 = require("../../../shim");
const notes_1 = require("./notes");
const Setting_1 = require("../../../models/Setting");
const fs_extra_1 = require("fs-extra");
const md5 = require('md5');
const imagePath = `${__dirname}/../../../images/SideMenuHeader.png`;
const jpgBase64Content = '/9j/4AAQSkZJRgABAQAAAQABAAD/2wBDAAMCAgICAgMCAgIDAwMDBAYEBAQEBAgGBgUGCQgKCgkICQkKDA8MCgsOCwkJDRENDg8QEBEQCgwSExIQEw8QEBD/wAALCAAFAAUBAREA/8QAFAABAAAAAAAAAAAAAAAAAAAACf/EAB8QAAEEAQUBAAAAAAAAAAAAAAQBAgUGAwAREiExM//aAAgBAQAAPwBJarVpGHm7KWbapCSwyZ6FDjkLyYE1W/LHyV2zfOk2TrzX/9k=';
describe('routes/notes', () => {
    beforeEach(() => {
        jest.resetAllMocks();
    });
    test.each([
        '/invalid/url',
        'htp/asdfasf.com',
        'https//joplinapp.org',
    ])('should not return a local file for invalid protocols', (invalidUrl) => __awaiter(void 0, void 0, void 0, function* () {
        yield expect((0, notes_1.downloadMediaFile)(invalidUrl)).resolves.toBe('');
    }));
    test.each([
        'https://joplinapp.org/valid/image_url.png',
        'http://joplinapp.org/valid/image_url.png',
    ])('should try to download and return a local path to a valid URL', (url) => __awaiter(void 0, void 0, void 0, function* () {
        const fetchBlobSpy = jest.fn((_url, options) => __awaiter(void 0, void 0, void 0, function* () {
            yield (0, fs_extra_1.writeFile)(options.path, Buffer.from(jpgBase64Content, 'base64'));
        }));
        const spy = jest.spyOn(shim_1.default, 'fetchBlob').mockImplementation(fetchBlobSpy);
        const response = yield (0, notes_1.downloadMediaFile)(url);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(1);
        expect(fetchBlobSpy).toHaveBeenCalledTimes(1);
        expect(response).toBe(`${Setting_1.default.value('tempDir')}/${files[0]}`);
        yield (0, fs_extra_1.remove)(response);
        spy.mockRestore();
    }));
    test('should get file from local drive if protocol allows it', () => __awaiter(void 0, void 0, void 0, function* () {
        const url = `file:///${imagePath}`;
        const originalFileContent = yield (0, fs_extra_1.readFile)(imagePath);
        const response = yield (0, notes_1.downloadMediaFile)(url, null, ['file:']);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(1);
        expect(response).toBe(`${Setting_1.default.value('tempDir')}/${files[0]}`);
        const responseFileContent = yield (0, fs_extra_1.readFile)(response);
        expect(md5(responseFileContent)).toBe(md5(originalFileContent));
        yield (0, fs_extra_1.remove)(response);
    }));
    test('should be able to handle URLs with data', () => __awaiter(void 0, void 0, void 0, function* () {
        const url = 'data:image/gif;base64,R0lGODlhEAAQAMQAAORHHOVSKudfOulrSOp3WOyDZu6QdvCchPGolfO0o/XBs/fNwfjZ0frl3/zy7////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAkAABAALAAAAAAQABAAAAVVICSOZGlCQAosJ6mu7fiyZeKqNKToQGDsM8hBADgUXoGAiqhSvp5QAnQKGIgUhwFUYLCVDFCrKUE1lBavAViFIDlTImbKC5Gm2hB0SlBCBMQiB0UjIQA7';
        const originalFileContent = Buffer.from(url.split('data:image/gif;base64,')[1], 'base64');
        const response = yield (0, notes_1.downloadMediaFile)(url);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(1);
        expect(response).toBe(`${Setting_1.default.value('tempDir')}/${files[0]}`);
        const responseFileContent = yield (0, fs_extra_1.readFile)(response);
        expect(md5(responseFileContent)).toBe(md5(originalFileContent));
        yield (0, fs_extra_1.remove)(response);
    }));
    test('should not process URLs with data that is not image type', () => __awaiter(void 0, void 0, void 0, function* () {
        const url = 'data:application/octet-stream;base64,dGhpcyBpcyBhIG1lc3NhZ2UK';
        Logger_1.default.globalLogger.enabled = false;
        const response = yield (0, notes_1.downloadMediaFile)(url);
        Logger_1.default.globalLogger.enabled = true;
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(0);
        expect(response).toBe('');
    }));
    test('should not process URLs from cid: protocol', () => __awaiter(void 0, void 0, void 0, function* () {
        const url = 'cid:ii_loq3d1100';
        const response = yield (0, notes_1.downloadMediaFile)(url);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(0);
        expect(response).toBe('');
    }));
    test('should not copy content from invalid protocols', () => __awaiter(void 0, void 0, void 0, function* () {
        const url = 'file:///home/user/file.db';
        const allowedProtocols = [];
        const mediaFilePath = yield (0, notes_1.downloadMediaFile)(url, null, allowedProtocols);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(0);
        expect(mediaFilePath).toBe('');
    }));
    test.each([
        'https://joplinapp.org/valid/image_url',
        'https://joplinapp.org/valid/image_url.invalid_url',
    ])('should correct the file extension in filename from files without or invalid ones', (url) => __awaiter(void 0, void 0, void 0, function* () {
        const spy = jest.spyOn(shim_1.default, 'fetchBlob').mockImplementation((_url, options) => __awaiter(void 0, void 0, void 0, function* () {
            yield (0, fs_extra_1.writeFile)(options.path, Buffer.from(jpgBase64Content, 'base64'));
            return {
                headers: {
                    'content-type': 'image/jpg',
                },
            };
        }));
        const response = yield (0, notes_1.downloadMediaFile)(url);
        const files = yield (0, fs_extra_1.readdir)(Setting_1.default.value('tempDir'));
        expect(files.length).toBe(1);
        expect(response).toBe(`${Setting_1.default.value('tempDir')}/${files[0]}`);
        yield (0, fs_extra_1.remove)(response);
        spy.mockRestore();
    }));
});
//# sourceMappingURL=notes.test.js.map